from abc import ABC, abstractmethod
from typing import List

from facenet_pytorch.models.mtcnn import MTCNN
import insightface
import numpy as np
import warnings

class VideoFaceDetector(ABC):

    def __init__(self, **kwargs) -> None:
        super().__init__()

    @property
    @abstractmethod
    def _batch_size(self) -> int:
        pass

    @abstractmethod
    def _detect_faces(self, frames) -> List:
        pass


class FacenetDetector(VideoFaceDetector):

    def __init__(self, device="cuda:0") -> None:
        super(FacenetDetector,self).__init__()

        self.detector = MTCNN(
            device=device,
            thresholds=[0.85, 0.95, 0.95],
            margin=0,
        )

    def _detect_faces(self, frames) -> List:
        """
        passes all the frames of a video and then returns the bboxes
        of each frame into batch_boxes.
        """
        batch_boxes, *_ = self.detector.detect(frames, landmarks=False)
        if batch_boxes is None:
            return []
        return [b.tolist() if b is not None else None for b in batch_boxes]
    @property
    def _batch_size(self):
        return 32

class FaceInfoExtractor(VideoFaceDetector):

    def __init__(self, device="cuda:0", need_embed=True, need_kps=True):
        super(FaceInfoExtractor,self).__init__()
        self.ret_model = insightface.app.FaceAnalysis(name="buffalo_l")
        if device == "cuda" or device == "cuda:0":
            device = 0
        elif device == "cpu":
            device = -1
        self.ret_model.prepare(ctx_id=device)
        self.need_embed = need_embed
        self.need_kps = need_kps
    def _extract_face_info(self, frame):
        details = self.ret_model.get(frame)
        faces_in_frame = []
        if len(details) == 0:
            warnings.warn("No faces detected in the frame")
            return []
        for detail in details:
            landmarks = detail.landmark_2d_106
            kps = detail.kps.tolist() if self.need_kps else np.zeros((1,))
            bbox = detail.bbox.tolist()
            embd = detail.embedding if self.need_embed else np.zeros((1,))
            faces_in_frame.append((bbox,kps,embd))
        del details
        return faces_in_frame
    def _detect_faces(self,frames) -> List:
        extracted_info = []
        for frame in frames:
            if not isinstance(frame, np.ndarray):
                warnings.warn("Frame is not a numpy array")
                continue
            extracted_info.append(self._extract_face_info(frame))
        return extracted_info
    @property
    def _batch_size(self):
        return 32


import os
import io
import sys
from contextlib import contextmanager
import json
import yaml
from tqdm import tqdm
import numpy as np
import torch
import cv2
from PIL import Image
from typing import Dict, List, Literal
from torchvision.transforms import PILToTensor, Resize
from identity_clustering.cluster import FaceCluster, cluster, detect_faces
from identity_clustering.utils import _get_frames, get_straight_face
@contextmanager
def suppress_stdout():
    saved_stdout = sys.stdout
    sys.stdout = io.StringIO()
    try:
        yield
    finally:
        sys.stdout = saved_stdout
def get_video_config(clustered_faces : Dict[int,list], identity : int, identity_dir : str | os.PathLike, filter_straight_face : bool = False, save_format : Literal["jpg", "png", "avi", "mp4"] = "jpg", device : str = "cpu", frame_rate : int = 30)->Dict[str,object]:
    config = {}
    config["ID"] = identity
    config["path"] = identity_dir
    config["class"] = "unassinged"
    exact_crop_dir = os.path.join(identity_dir, "exact_crops")
    if not os.path.exists(exact_crop_dir):
        os.makedirs(exact_crop_dir)
    pil_to_tensor = PILToTensor()
    resize = Resize((512, 512))
    # bboxes = []
    writer_1 = None
    writer_2 = None
    box = {}
    exact_crop_faces_stack = []
    faces_stack = []
    bbox,kps,embed=None,None,None
    for frame, cropped_face, vid_info_bbox, exact_crop in clustered_faces[identity]:
        if not isinstance(vid_info_bbox, list):
            bbox, kps, embed = vid_info_bbox
        else:
            bbox = vid_info_bbox
        if save_format in ["jpg", "png"]:
            if not filter_straight_face or (filter_straight_face and kps and get_straight_face(kps)):
                face_path = os.path.join(identity_dir,f"{frame}.{save_format}")
                # image_cv = cv2.cvtColor(np.array(cropped_face), cv2.COLOR_BGR2RGB)
                # exact_crop = cv2.cvtColor(np.array(exact_crop), cv2.COLOR_BGR2RGB)
                # cropped_face = Image.fromarray(image_cv)
                exact_crop_face_path = os.path.join(exact_crop_dir,f"{frame}.{save_format}")
                # exact_crop = Image.fromarray(exact_crop)
                cropped_face.save(face_path)
                exact_crop.save(exact_crop_face_path)
        elif save_format in ["avi", "mp4"]:
            face_np = resize(pil_to_tensor(cropped_face)).permute(1, 2, 0).cpu().numpy()
            exact_np = resize(pil_to_tensor(exact_crop)).permute(1, 2, 0).cpu().numpy()
            if face_np.ndim == 2:
                face_np = cv2.cvtColor(face_np, cv2.COLOR_GRAY2BGR)
            if exact_np.ndim == 2:
                exact_np = cv2.cvtColor(exact_np, cv2.COLOR_GRAY2BGR)

            face_bgr = cv2.cvtColor(face_np, cv2.COLOR_RGB2BGR)
            exact_bgr = cv2.cvtColor(exact_np, cv2.COLOR_RGB2BGR)

            # Lazily initialize writers using first valid frame
            if writer_1 is None:
                h1, w1 = face_bgr.shape[:2]
                h2, w2 = exact_bgr.shape[:2]

                fourcc = cv2.VideoWriter_fourcc(*"mp4v")
                writer_1 = cv2.VideoWriter(
                    os.path.join(identity_dir, f"{identity}.{save_format}"),
                    fourcc,
                    frame_rate,
                    (w1, h1)
                )
                writer_2 = cv2.VideoWriter(
                    os.path.join(exact_crop_dir, f"{identity}.{save_format}"),
                    fourcc,
                    frame_rate,
                    (w2, h2)
                )

            writer_1.write(face_bgr)
            writer_2.write(exact_bgr)
            # exact_crop_faces_stack.append(pil_to_tensor(exact_crop).to(device))
            # faces_stack.append(pil_to_tensor(cropped_face).to(device))
            
        box[frame] = bbox
        with open(os.path.join(identity_dir,f"{frame}.json"),"w") as f:
            save = {frame:bbox}
            if not kps is None:
                save["kps"] = kps
            json.dump(save,f)
        if (not (embed is None)) and isinstance(embed,np.ndarray) and embed.shape==(512,):
            file = os.path.join(identity_dir,f"{frame}.npy")
            np.save(file,embed)
    config["bboxes"] = box
    # if save_format in ["avi", "mp4"] and False: # remove false after save_format
    #     np.save(np.array(config["bboxes"].keys()), os.path.join(identity_dir,f"{identity}_frames.npy"))
    #     faces_stack = torch.stack(faces_stack)
    #     exact_crop_faces_stack = torch.stack(exact_crop_faces_stack)
    #     encoder = VideoEncoder(faces_stack, frame_rate = frame_rate)
    #     encoder.to_file(os.path.join(identity_dir,f"{identity}.{save_format}"))
    #     encoder = VideoEncoder(exact_crop_faces_stack, frame_rate = frame_rate)
    #     encoder.to_file(os.path.join(exact_crop_dir,f"{identity}_exact_crop.{save_format}"))
    if save_format in ["avi", "mp4"]:
        if writer_1 is not None:
            writer_1.release()
        if writer_2 is not None:
            writer_2.release()
    return config
def jsonify(dataset_root_path: str | os.PathLike, 
            project_name : str,
            save_path : str | os.PathLike,
            save_embed : bool = False,
            filter_straight_face : bool = False,
            save_kps : bool = False,
            save_landmarks : bool = False,
            save_format : Literal["png", "jpg", "avi", "mp4"] = "jpg",
            save_codec : Literal["libx264", "h264", None] = None,
            frame_rate : int = 30,
            padding_constant : int = 3,
            multiplier : int = 2,
            similarity_threshold : float = 0.9,
            max_identities_limit : int = 6,
            device : str = "cpu"):
    '''
        dataset_root_path: str - Path to the videos in the dataset
        project_name: str - Name of the project. This name will be used to create a folder in which all the extracted information will be stored.
        save_path: str - Path to which the generated results will be saved
        save_format : Literal["png", "jpg", "avi", "mp4"] - processed frames can be stored as frames or videos per identity, this is introduced with the torchcodec update made to the identity_clustering package
        save_codec : Literal["libx264", "h264", None] - refers to the video codec to be used while saving the video.
        frame_rate : int - frame rate of the video.
        save_embed : bool - saves arcface (insightface webface_r50.onnx) embeddings of the face.
        save_kps : bool - saves the key landmarks of the face, eyes, nose and the two ends of the mouth
        save_landamrks : bool - saves the 81 face landmark from dlib (exp)
        filter_straight_face : bool - filters straight face i.e. videos with no straight face is dropped (not recomm, always set to False)
        max_identities_limit : int - clustered identities can sometimes exceed the actual identities in the video because of poor model performance, so to cap the number of identities to be saved one can use this. default 6. (recommended)

        Here is the folder structure:
                | PROJECT_NAME
                    | VIDEO 1
                        | IDENTITY 1
                            FRAME_NO.jpg
                            FRAME_NO.jpg
                        | IDENTITY 2
                            FRAME_NO.jpg
                            FRAME_NO.jpg
                        ....
                        ....
                        ....
                        | IDENTITY N
                            FRAME_NO.jpg
                            FRAME_NO.jpg
                        PROJECT_NAME.json
                    ....
                    ....
                    ....
                    | VIDEO N
                        | IDENTITY 1
                            FRAME_NO.jpg
                            FRAME_NO.jpg
                        | IDENTITY 2
                            FRAME_NO.jpg
                            FRAME_NO.jpg
                        ....
                        ....
                        ....
                        | IDENTITY N
                            FRAME_NO.jpg
                            FRAME_NO.jpg
                        PROJECT_NAME.json
    '''
    try:
        '''
            - The dictionary below will follow the template as mentioned in the above markdown cell.
            - It will contain information about the dataset, the videos in the dataset, the extracted identities, the path in which the identities'
              cropped face images will be stored and finally the bounding box information of the faces of each identity in the dataset.
            - The dictionary will ultimately be stored as a JSON file.
        '''
        videos_list = os.listdir(dataset_root_path)
        dataset_json = {}
        dataset_json["root_path"] = dataset_root_path
        dataset_json["project"] = project_name
        '''
            - The below videos array will be a part of dataset_json['videos']. Refer to the videos key in the above markdown.
            - This array will contain all the information about the videos in the dataset. It is an array of objects. Each object will refer to one video in
            the dataset. Inside each object, you'll find information about the path to the video, the fps of the video, and information regarding each
            identity in the video.
            - Intialize the Clustere class for clustering identities.
        '''
        # videos = []
        clust = FaceCluster() # type: ignore
        '''
            Refer to the comment at the start of the function for the folder structure.
            Now let's create a folder in which we will store the JSON file, the cropped faces of each identity in the videos.
            For reference, this folder is the root folder PROJECT_NAME
        '''
        project_path = os.path.join(save_path,project_name)
        if not os.path.exists(project_path):
            os.mkdir(project_path)
        '''
            Now let's go through every video in the dataset and process them to get the identities. Here are the steps:
                1. For each video, extract all the faces in every frame. Store them in an array.
                2. Cluster the faces. Each cluster is an identity, i.e, faces of a person.
                3. Store them in a folder.
            While doing each step, let's log them and also store the required information in the dataset_json
        '''
        error_directory = os.path.join(project_path,"error_logs")
        if not os.path.exists(error_directory):
            os.mkdir(error_directory)
        for video_name in tqdm(sorted(videos_list)):
            try:
                torch.cuda.empty_cache()
                '''
                    video_config: dict - It is a dictionary that goes into the videos array. It contains information about the identities in the video and other
                    meta information about the video. Refer to the object in videos key in the above JSON.
                '''
                video_config = {}
                '''
                    video_name: str - Full path to the video.
                    destination_path: str - path to the folder in which the identities' cropped faces will be stored.
                '''
                video_path = os.path.join(dataset_root_path,video_name)
                destination_path = os.path.join(project_path, video_name)
                ### Create a folder for the video, if the folder exists skip that video
                if not os.path.exists(destination_path):
                    os.mkdir(destination_path)
                else:
                    continue
                video_config["path"] = video_path
                #extract faces
                faces,fps=None,None
                with suppress_stdout():
                    if save_embed and filter_straight_face:
                        faces,fps = detect_faces(video_path,"cuda",v2=True)
                    elif filter_straight_face:
                        faces,fps= detect_faces(video_path,"cuda",v2=True,need_embed=False)
                    elif save_embed:
                        faces,fps= detect_faces(video_path,"cuda",v2=True,need_kps=False)
                    else:
                        faces, fps = detect_faces(video_path, "cuda:0")
                video_config["fps"] = fps
                clustered_faces = cluster(clust, video_path, faces, padding_constant, similarity_threshold, multiplier)
                if len(clustered_faces) > max_identities_limit:
                    continue
                ### Identity means one person in a video (clustered faces)
                '''
                    identities: [
                        {
                            path: str - path to the face crops of a single person,
                            ID: int - a unique number attached to that person,
                            class: str - enum(fake/real)
                        }
                    ]
                '''
                ### Identities is a collection of the faces of all the people in a video
                # Identities = []
                ###
                for identity in list(clustered_faces.keys()):
                    if not clustered_faces[identity]:
                        ### If no faces in a specific cluster
                        continue
                    ### Create a folder for each identity
                    identity_dir = os.path.join(destination_path,str(identity)) ## Stored ad path in the array
                    if not os.path.exists(identity_dir):
                        os.mkdir(identity_dir)
                    with suppress_stdout():
                        config = get_video_config(clustered_faces, identity, identity_dir, filter_straight_face, save_format, device, frame_rate)
                    with open(os.path.join(identity_dir,f"{identity}_info.json"),"w") as f:
                        json.dump(config,f)
                #     identities.append(config)
                # video_config["Identities"] = identities
                with open(os.path.join(destination_path,f"{video_name}.json"),"w") as f:
                        json.dump(video_config,f)
            except Exception as e:
                with open(os.path.join(error_directory,f"{video_name}.txt"),"w") as f:
                    f.write(str(e))
                print(str(e))
                continue
        torch.cuda.empty_cache()
        with open(os.path.join(project_path,f"{project_name}.json"),"w") as f:
            json.dump(dataset_json,f)
    except Exception as e:
        raise Exception(e)
if __name__ == "__main__":
    # /mnt/c/Users/ASUS/Desktop/PhospheneAI2/pkg-face-clustering/
    with open("/mnt/c/Users/ASUS/Desktop/PhospheneAI2/pkg-face-clustering/identity_clustering/data_proc/config/jsonify.yaml",'r') as file:
        config_jsonify = yaml.safe_load(file)
    #Path to store the JSON file and Some extracted Images
    SAVE_PATH = config_jsonify["SAVE_PATH"]
    #setting Two main JSON attributes the root path and the project name, the root path will
    ROOT_PATH =   config_jsonify["ROOT_PATH"]
    # MASK_PATH = config_jsonify["MASK_PATH"]
    PROJECT_NAME = config_jsonify["PROJECT_NAME"]
    save_embed = config_jsonify["save_embed"]
    filter_straight_face = config_jsonify["straight_face"]
    #device
    device = torch.device(config_jsonify["DEVICE"])
    jsonify(ROOT_PATH, PROJECT_NAME,SAVE_PATH, save_embed, filter_straight_face, save_format="jpg", device=device, frame_rate=10)


from collections import OrderedDict
import cv2
from PIL import Image
from torch.utils.data.dataloader import Dataset
import numpy as np
import warnings
from torchcodec.decoders import VideoDecoder
from torchvision.transforms import Resize

class VideoDataset(Dataset):
    """
    Dataset class for fetching specific information about a video.
    The class requires a list of videos that it has to process.
    """

    def __init__(self, videos, v2=False, device='cuda') -> None:
        super().__init__()
        self.videos = videos
        self.v2 = v2
        self.device = device
    def __getitem__(self, index: int):
        """
        This function, picks a video and returns 4 values.
            str: Contains the name of the video
            list: List containing only the frame indices that are readable
            number: fps of the video
            list: a list containing all the frames as image arrays
        """
        video = self.videos[index]
        decoder = VideoDecoder(video, device=self.device)
        frames_num = decoder.metadata.num_frames
        fps = decoder.metadata.average_fps_from_header
        height, width = decoder.metadata.height, decoder.metadata.width
        frames = decoder[:]
        if not self.v2:
            resize = Resize((height//2, width//2))
            frames = resize(frames)

        frames = frames.permute(0, 2, 3, 1).cpu().numpy()
        if self.v2 and len(frames) > 0 and ((not isinstance(frames[0], np.ndarray) and (not isinstance(frames[0], Image.Image)))):
            warnings.warn("No Frames read in the video")
        return video, range(frames_num), fps, frames

    def __len__(self) -> int:
        return len(self.videos)



from pathlib import Path

# def get_version():
#     script_dir = Path(__file__).resolve().parent
#     pyproject_path = script_dir / "../pyproject.toml"
#     pyproject_path = pyproject_path.resolve()
#     with open(pyproject_path, "r") as f:
#         pyproject = toml.load(f)
#     return pyproject["tool"]["poetry"]["version"]


__version__ = "2.2.2"



from typing import Dict, List

import networkx as nx
import numpy as np
import torch
from facenet_pytorch import InceptionResnetV1
from torch.utils.data.dataloader import DataLoader
from torchvision.transforms import Resize, ToTensor, Compose
from PIL import Image
import os
from identity_clustering.dataset import VideoDataset
from identity_clustering.facedetector import FacenetDetector, FaceInfoExtractor
from identity_clustering.utils import extract_crops


class FaceCluster:

    def __init__(
            self, crops=None, similarity_threshold: int = 0.85, device: str = "cpu", allow_single_face_cluster: bool = False, is_profiling : bool = False, is_inference : bool = False, shape:tuple = (128,128),
            arcface_model_path : str | os.PathLike = None, arcface : bool = False
    ):
        import tempfile
        self.similarity_threshold = similarity_threshold
        self.device = torch.device(device)
        self.crops = crops 
        self.shape = shape
        self.is_inference = is_inference
        self.tempdir = tempfile.TemporaryDirectory()
        self.save_path = os.path.join(self.tempdir.name,"webface_r50.onnx")
        self.is_external_path_provided = False
        if arcface_model_path:
            self.is_external_path_provided = True
            if os.path.isdir(arcface_model_path):
                self.save_path = os.path.join(arcface_model_path,"webface_r50.onnx")
            else:
                self.save_path = arcface_model_path
        self.embedding_extractor = InceptionResnetV1(pretrained="vggface2").eval().to(self.device)
        self.url = "https://drive.usercontent.google.com/download?id=1N0GL-8ehw_bz2eZQWz2b0A5XBdXdxZhg&export=download&authuser=0&confirm=t&uuid=ac614f9a-9111-4a83-80c3-6dea200a6819&at=ALoNOgl4wR5G2t7y0FvEgdkxJISP%3A1748804040944"
        self.headers = {
            "Host": "drive.usercontent.google.com",
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/137.0.0.0 Safari/537.36",
            "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.7",
            "Accept-Language": "en-US,en;q=0.9",
            "Cookie": "HSID=AAj_x9vXkihL9Wl9Z; SSID=AIfz0uEnXkliBvcK8; APISID=kbgv6u2tlFych_Xr/AJ4JNwhja4T19yItr; SAPISID=kini1tsMuoqYlEOb/AgN4kdzVHGsjDe9pm; __Secure-1PAPISID=kini1tsMuoqYlEOb/AgN4kdzVHGsjDe9pm; __Secure-3PAPISID=kini1tsMuoqYlEOb/AgN4kdzVHGsjDe9pm; AEC=AVcja2eVApAUPUzOftIotspLVp1IMhw3JuJJMnkgxOV2tK4G00dPNkFnO8M; __Secure-ENID=28.SE=kSIBR3PxTal8ix_r69VXmQKa8rs_hcM3qF1-93E2MGvcS20Qz-OCx9KuyGh949IAUTTwb4cPZNOzPY252n1YkozKWiIEOaF7VfyjAIX64aebwdbPg_6zk1uFYSNalScjefZADI6-yz33yPdXrhsi994XArloINd31lg7lAMs78Ryu94gnr-4bfhSasPeqPfSyGegzyXCj7SkNqQAC2xP7fo09w6bZt09bRhtjCWAX7z-sOs; SID=g.a000xgjwjpJCTOAYkiI1p46Jy5WLY1dcq_Qk3o1O-XVrI-F25XnZ6Rj5y9PQ8bOEI9ONjpPxegACgYKAeYSARASFQHGX2MiGDwHX9rrwap652xZENzjBxoVAUF8yKplSmVrxy7RA8rKkmE0DiIX0076; __Secure-1PSID=g.a000xgjwjpJCTOAYkiI1p46Jy5WLY1dcq_Qk3o1O-XVrI-F25XnZdtrdI_skkHuBB4e3PaV3GwACgYKAQcSARASFQHGX2MiSBAPwysTR5T-jzaAcCNnmBoVAUF8yKqRlur3_p8SGS2Sf5wLJyco0076; __Secure-3PSID=g.a000xgjwjpJCTOAYkiI1p46Jy5WLY1dcq_Qk3o1O-XVrI-F25XnZC3tUsnbAHnWOcSTVf1_bngACgYKAbMSARASFQHGX2MievIUdQ8AcwyK3rY_FB8oFxoVAUF8yKoLotLjezpFFzLW9b-LnQAx0076; NID=524=ZlLiwZ7sGQUsyranW44RKFNN-5jH5tmsDwzBEwPmReJjNzezcPxAa2gWJ56-iXT2TFGyTstBaru5SjswZw3Zcihs7oHMN-c3kWKOrH-BnT2xPrcvE0-0CEXMZ4TWe_3Z9E0tFGEZfReIswlp5gKsq6ktHvhv4H108JqUa7jLCOB6P0w0qCCRDPJenC8XYHlKI1WpfQ_fJu82ejkn64ClycIVpasv8fhcALyzDB3Phn5032ZucxD7H6l4WQPMaMYFj0UTTmfdd1fMBWps5NkuWbw-wf1xpcLSxOfGGe3EK1x1CHUcsZZ0ZJ8Qsje95iOm0fhlqYZmnio7YWoynMJ_FWqLX7tCMkTIFub3cScRCUVacQCM_R9nY49MARNPCPUIIuV0-sXbCRnHXem4CBQLd6sYA1JgwLpD7TH-J9CapnUBzeK-Z-m8OKX4WCu3q0WrDXpHnrjMl316OlzppRGHBHTV1QCTzohLANx9RUQe2nJ6XxQMrkf95IgA1J62avn81g5oN-oR9mgZzH1V2HxHP5eUDuObFa_CHz9pvoUjaGi9EqN4owM59-2IUL7P5WSnf9ZN87exthQfH5JxA450dOhblRtsj1Q1r1vxqdHbUI2QLWtPt0iwkPIkjyzUzA4sQvLkIBgTZa2iANZLowgFudq1OCSDvTGs_J2U9-awBNW3NwiyrdoTNy0jq1FEAnuEKdxeBNlrOxJ6aXbaUYycIAqXSxWvhUPG0tX52aVxRatNLMIBEVGG66FWlRYTSBBi8Wbs3Zbjt1nTSn0u5AGLTx2nqV8ZNZ2mUqBPG9bXO5kHDfTEkcegnu5B5gbsQGTDCpStcoYF2_e49SYH2FSchSNn7ciulHrQSJgxzX_LOHWzxtjC5Di5KEf1sbdX8-2XTnb2rTKmz0hPO8to-0S9yKy-9UshubIUZ4ftM2i7Bz6YEWwcnDZuGDm1doPvd7CYBB8tkazSlgawIl4m-igOXs-mol_wT2sn09TVMwiys9UrlbrEjuupwrv-m04CrMNAjYSPPpN8ZbtHb4Nibd2alT8XcvKsZvg23qzf19GNo1VbzHs; __Secure-1PSIDTS=sidts-CjEB5H03Pwouwt_2gm8il5It9bTxqGgrqrreWDmzQopuCUppITOFgRcked6FLgH5zrs_EAA; __Secure-3PSIDTS=sidts-CjEB5H03Pwouwt_2gm8il5It9bTxqGgrqrreWDmzQopuCUppITOFgRcked6FLgH5zrs_EAA; SIDCC=AKEyXzVLdNWidgn3FSzfTNgWxKPf-DGgVVBi6WUOlzGMJ1QtVk8WR4PBDLoiWUqjIDqnWBPB2uUc; __Secure-1PSIDCC=AKEyXzVqZPA-FcYMdNDHDBh1k1xAwzyW6PSUMrbQQ4XUlewpvLH-LBH1n_YyZOJGV0PuPMHidmqh; __Secure-3PSIDCC=AKEyXzU9T2QhcI2Wks2F9FEJ07fvJkTs_Wt6dburMNakc6khQf4uiXKavAEsdlicf6Oh5stqapz2",
            "Connection": "keep-alive"
        }
        self.arcface = arcface
        if self.arcface:
            self.use_arcface()
    def __enter__(self):
        return self

    def __exit__(self,exc_type, exc_value, traceback):
        self.clean_up()

    def use_arcface(self):
        '''
        if using without with statement call this function to use arcface model at the begining. don't forget to call clean up later.
        '''
        import onnx
        from onnx2torch import convert
        import warnings
        import requests
        import tqdm
        if not os.path.exists(self.save_path):
            response = requests.get(self.url, stream=True, headers=self.headers)
            total_size = int(response.headers.get("content-length", 0))

            with open(self.save_path, "wb") as file, tqdm.tqdm(
                desc=self.save_path,
                total=total_size,
                unit="B",
                unit_scale=True,
                unit_divisor=1024,
            ) as bar:
                for data in response.iter_content(chunk_size=1024):
                    size = file.write(data)
                    bar.update(size)
        if os.path.exists(self.save_path):
            onnx_model = onnx.load(self.save_path)
            model = convert(onnx_model)
            self.embedding_extractor = model.eval().to(self.device)
            self.shape = (112,112)
        else:
            warnings.warn(f"Arcface weights download failed, falling back to InceptionNetv2(vggface)")

    def clean_up(self):
        import gc
        if not self.is_external_path_provided and os.path.exists(self.save_path):
            os.remove(self.save_path)
        self.tempdir.cleanup()
        del self.embedding_extractor
        gc.collect()
        torch.cuda.empty_cache()
    def _set_crops(self, crops) -> None:
        '''
        A setter function to set the threshold attribute
        Args :
         - crops : List[tuple] -> contains tuples with (frame_no(int), PIL Image of the cropped face, bbox(list(int)))

        Returns :
            None
        '''
        self.crops = crops

    def _set_threshold(self, threshold : float) -> None:
        '''
        A setter function to set the threshold attribute
        Args :
         - threshold : float -> threshold value.

        Returns :
            None
        '''
        self.similarity_threshold = threshold

    def _generate_connected_components(self, similarities):
        '''
        Helper function for the clustering function, takes in dot product similarities and clusters them based on a predefined threshold,
        the threshold can be set by the user when intializing the class or can be set while calling the `cluster_faces` function
        
        Args :
         - similarities : np.ndarray -> similarity matrix (attention without scaling)

        Returns :
         - components : list -> list of clustered faces
        '''
        graph = nx.Graph()
        n = similarities.shape[0]
        threshold = self.similarity_threshold
        mask = (similarities > threshold) & (~np.eye(n, dtype = bool))
        edges = np.argwhere(mask)
        graph.add_edges_from(map(tuple,edges))
        components_list = [list(component) for component in nx.connected_components(graph)]
        return components_list
    @torch.no_grad
    def get_similarities_on_images(self, crops_images):
        '''
        Being Kept for compatibility
        '''
        transform = Compose([Resize(self.shape), ToTensor()])
        faces = torch.stack([transform(face) for face in crops_images])
        faces = faces.to(self.device, non_blocking=True)
        embeddings = self.embedding_extractor(faces)
        if self.arcface:
            embeddings = torch.nn.functional.normalize(embeddings, p=2)
        similarities = torch.mm(embeddings, embeddings.T).cpu().numpy()
        return similarities
    def get_similarities_on_embeds(self, crops_embeds):
        embeddings = torch.stack([torch.tensor(face) for face in crops_embeds])
        embeddings = embeddings.to(self.device, non_blocking=True)
        embeddings = torch.nn.functional.normalize(embeddings, p=2)
        similarities = torch.mm(embeddings, embeddings.T).cpu().numpy()
        return similarities
    def cluster_faces(self, crops, threshold=None):
        '''
        Function that clusters the faces using the dot product similarity metric.
        
        Args:
         - crops : List[tuple] -> contains tuples with (frame_no(int), PIL Image of the cropped face, bbox(list(int)))
         - threshold : Optional[float] -> set to change the threshold attribute.
        
        Returns :
         - clustered_faces : Dict[int,list] -> returns a dictionary containing the identity as keys and all the faces associated with a single
                                               identity as a list.
        '''

        if threshold:
            self._set_threshold(threshold)

        if crops and self.is_inference:
            self._set_crops(crops)

        # Convert crops to PIL images
        try:
            assert isinstance(crops[0][2][-1], np.ndarray) and crops[0][2][-1].shape == (512,)
            crops_embeds = [row[2][-1] for row in crops]
            similarities = self.get_similarities_on_embeds(crops_embeds)
        except AssertionError:
            assert isinstance(crops[0][3], Image.Image)
            crops_images = [row[3] for row in crops]
            similarities = self.get_similarities_on_images(crops_images)
        
        components = self._generate_connected_components(similarities)
        components = [sorted(component) for component in components]

        # Assigning each cluster to a unique identity
        clustered_faces = {}
        for identity_index, component in enumerate(components):
            for index, face_index in enumerate(component):
                component[index] = crops[face_index]

            clustered_faces[identity_index] = component



        return clustered_faces


def cluster(
    clust: FaceCluster,
    video_path: str,
    faces: List[tuple],
    pad_constant: int | tuple | None = 3,
    similarity_threshold : float = 0.9,
    multiplier : int = 2 # NOTE: new changes made with torchcodec makes how the mulitplier work in the _get_crop func a bit mysterious, must be investigated later.
) -> Dict[int, list]:
    crops = extract_crops(video_path, faces, pad_constant, multiplier)
    clustered_faces = clust.cluster_faces(crops, similarity_threshold)
    return clustered_faces

def detect_faces(video_path, device, need_embed=True, need_kps=True, v2=False):
    """  
    We'll be using the facenet detector that is required to detect the faces
    present in each frame. This function is only responsible to return
    a dictionary that contains the bounding boxes of each frame.
    Args:
        video_path: str - Path to the video.
        device: str - indicates whether to leverage CPU or GPU for processing.
        need_embed: bool - set to true if you wish to save arcface embeddings of faces in frames.
        need_kps: bool - set to true if you want kps points for each face.
        v2: bool - set to true to make sure the VideoDataset class loads data in the proper format.
    returns: 
        dict: dict template:
            {
                frame_no: [[
                    [number, number, number, number],
                    [number, number, number, number],
                    ...
                    ...
                    [number, number, number, number]
                ]]
            }
        int: fps of the video
    """
    detector = FacenetDetector(device=device)
    if v2:
        detector = FaceInfoExtractor(device=device, need_embed=need_embed, need_kps=need_kps)

    # Read the video and its information
    dataset = VideoDataset([video_path],v2=v2)
    # loader = DataLoader(
    #     dataset, shuffle=False, num_workers=0, batch_size=1
    # )
    import unittest
    # Detect the faces
    for item in dataset:
        bboxes = {}
        video, indices, fps, frames = item
        # assert len(frames) > 0 and (isinstance(frames[0], np.ndarray) or isinstance(frames[0], Image.Image)) or isinstance(frames[0],unittest.mock.MagicMock), f"No Frames read in the video or the frames are not in the correct type {type(frames[0])}"
        """
            Update bboxes dict with the bounding boxes present in each frame with the 
            frame number as the index and 
            a two dimensional list containing the bounding boxes as the value. 
        """
        bboxes.update({i: b for i, b in zip(indices, detector._detect_faces(frames))})
        found_faces = False
        for key in list(bboxes.keys()):
            if isinstance(bboxes[key], list):
                found_faces = True
                break

        if not found_faces:
            return None, indices[-1]
    return bboxes, fps


from abc import ABC, abstractmethod
from collections import OrderedDict
from typing import Dict, List

import cv2
import networkx as nx
import numpy as np
import torch
from facenet_pytorch import InceptionResnetV1
from facenet_pytorch.models.mtcnn import MTCNN
from PIL import Image
from torch.utils.data.dataloader import Dataset
from torchvision.transforms import Resize



import cv2
from PIL import Image
import numpy as np
from torchcodec.decoders import VideoDecoder
from sys import getsizeof
def _get_frames(video_path):
    """
    This function gets the video path, reads the video, stores
    the frames in a list and then returns
    """
    decoders = VideoDecoder(video_path, device = 'cuda')
    frames = decoders[:].permute(0,2,3,1).cpu().numpy() #TO FIX: match pil format
    return frames


def _get_crop(frame, bbox, pad_constant : int | tuple = 3, multiplier: int = 2):
    '''
        This function takes a frame and a bbox and then outputs the region of the image given by the bounding box
        Args : 
        - frame : np.ndarray -> image frame containing the faces to be cropped.
        - bbox : list -> the bounding box associated with that frame.
        - pad_constant : int -> The constant to control the padding. Default is None.
        - use_pad_constant : bool -> If True, uses the pad_constant to control the padding. Default is False.
        - multiplier : int -> mulitplier to multiply bbox values (must be clearly defined later) (DO NOT CHANGE THE MULTIPLIER)
        Returns :

        - crop : np.ndarray -> the cropped output of the faces.
    '''
    xmin, ymin, xmax, ymax = [int(b*multiplier) for b in bbox]
    w = xmax - xmin
    h = ymax - ymin

    # Add some padding to catch background too
    '''
                          [[B,B,B,B,B,B],
    [[F,F,F,F],            [B,F,F,F,F,B],
     [F,F,F,F],    --->    [B,F,F,F,F,B],
     [F,F,F,F]]            [B,F,F,F,F,B],
                           [B,B,B,B,B,B]]

            F -> Represents pixels with the Face.
            B -> Represents the Background.
            padding allows us to include some background around the face.
            (padding constant 3 here causes some issue with some videos)
    '''
    p_w = 0
    p_h = 0
    if type(pad_constant) == int:
        p_h = h // abs(pad_constant)
        p_w = w // abs(pad_constant)
    elif type(pad_constant) == float:
        p_h = h // abs(pad_constant[0])
        p_w = w // abs(pad_constant[1])

    
    crop_h = (ymax + p_h) - max(ymin - p_h, 0)
    crop_w = (xmax + p_w) - max(xmin - p_w, 0)

    # Make the image square
    '''
    Makes the crop equal on all sides by adjusting the pad
    '''
    if crop_h > crop_w:
        p_h -= int(((crop_h - crop_w)/2))
    else:
        p_w -= int(((crop_w - crop_h)/2))

    # Extract the face from the frame
    crop = frame[max(ymin - p_h, 0):ymax + p_h, max(xmin - p_w, 0):xmax + p_w]
    
    # Check if out of bound and correct
    h, w = crop.shape[:2]
    if h > w:
        diff = int((h - w)/2)
        if diff > 0:         
            crop = crop[diff:-diff,:]
        else:
            crop = crop[1:,:]
    elif h < w:
        diff = int((w - h)/2)
        if diff > 0:
            crop = crop[:,diff:-diff]
        else:
            crop = crop[:,:-1]

    return crop
def extract_crops(video_path, bboxes_dict, pad_constant: int | tuple = 3, multiplier :int = 2):

    frames = _get_frames(video_path)
    crops = []
    keys = [int(x) for x in list(bboxes_dict.keys())]
    for i in range(0, len(frames)):
        frame = frames[i]
        if i not in keys:
            continue
        bboxes = bboxes_dict[i]
        if not bboxes:
            continue
        for bbox in bboxes:
            if isinstance(bbox, list):
                crop = _get_crop(frame, bbox, pad_constant, multiplier)
                fram = _get_crop(frame, bbox, 8, multiplier)
            else:
                crop = _get_crop(frame, bbox[0], pad_constant,multiplier=1)
                fram = _get_crop(frame, bbox[0], 8,multiplier=1)
            # Add the extracted face to the list
            crops.append((i, Image.fromarray(crop), bbox, Image.fromarray(fram)))

    return crops

def draw_faces(frame, faces):
    for bbox, kps, embedding in faces:
        x1, y1, x2, y2 = map(int, bbox)
        cv2.rectangle(frame, (x1, y1), (x2, y2), (0, 255, 0), 2)
        for (x, y) in kps:
            cv2.circle(frame, (int(x), int(y)), 2, (0, 0, 255), -1)

    return frame

def get_straight_face(kps,lowerbound=0.85,upperbound=1.25):
    if len(kps)<5:
        return False
    len_ = np.sqrt((kps[0][0] - kps[2][0])**2 + (kps[0][1] - kps[2][1])**2)
    ren_ = np.sqrt((kps[1][0] - kps[2][0])**2 + (kps[1][1] - kps[2][1])**2)
    lmn_ = np.sqrt((kps[3][0] - kps[2][0])**2 + (kps[3][1] - kps[2][1])**2)
    rmn_ = np.sqrt((kps[4][0] - kps[2][0])**2 + (kps[4][1] - kps[2][1])**2)
    pitch_ratio = (len_ + ren_)/(lmn_ + rmn_)
    yaw_ratio = (len_+lmn_)/(ren_+rmn_)
    print(pitch_ratio, yaw_ratio)
    if not lowerbound < pitch_ratio< upperbound or not lowerbound < yaw_ratio < upperbound:
        return False
    return True



import os
import sys
import time
from identity_clustering.cluster import detect_faces
from identity_clustering.utils import extract_crops, _get_frames, _get_crop
from streaming import MDSWriter
from typing import Dict, List, Literal, Optional
import multiprocessing as mp
import subprocess
import tempfile
import tqdm
import numpy as np
import shutil
import argparse


def get_max_h_and_w(crops: List[np.ndarray]) -> tuple[int, int]:
    """
    Compute the maximum height and width across a list of image crops.

    Args:
        crops (List[np.ndarray]): List of image crops with shape (H, W, C).

    Returns:
        tuple[int, int]: Maximum height and maximum width.
    """
    max_h, max_w = -1, -1
    for i in crops:
        max_h = max(max_h, i.shape[0])
        max_w = max(max_w, i.shape[1])
    return max_h, max_w


def pad_crops_to_max_dims(
    crops: List[np.ndarray],
) -> tuple[List[np.ndarray], np.ndarray, int, int]:
    """
    Pad all crops to the maximum height and width among them.

    Args:
        crops (List[np.ndarray]): List of image crops.

    Returns:
        Tuple containing:
            - List[np.ndarray]: Padded crops
            - np.ndarray: Original (H, W) for each crop
            - int: Maximum height
            - int: Maximum width
    """
    max_h, max_w = get_max_h_and_w(crops)
    new_crops = []
    actual_h_w = []
    for crop in crops:
        new_crop = np.zeros((max_h, max_w, crop.shape[-1]), dtype=crop.dtype)
        h_, w_ = crop.shape[0], crop.shape[1]
        new_crop[:h_, :w_, :] = crop
        new_crops.append(new_crop)
        actual_h_w.append(np.array([h_, w_]))
    actual_h_w = np.vstack(actual_h_w)
    return new_crops, actual_h_w, max_h, max_w


def upload_to_s3(
    s3_uri: str,
    local_dir: str | os.PathLike,
    upload_interval: int,
    log_dir: Optional[str | os.PathLike],
    stop_event: mp.Event
) -> None:
    """
    Periodically sync local shard directory to S3 until stop signal is received.

    Args:
        s3_uri (str): Destination S3 URI.
        local_dir (str | os.PathLike): Local shard directory.
        upload_interval (int): Interval between syncs in seconds.
        log_dir (Optional[str | os.PathLike]): Directory to store uploader logs.
        stop_event (mp.Event): Event signaling uploader termination.
    """
    if log_dir is None:
        stderr = subprocess.DEVNULL
    else:
        os.makedirs(log_dir, exist_ok=True)
        stderr = open(os.path.join(log_dir, "s3_upload_err_logs.log"), "ab")

    while not stop_event.is_set():
        subprocess.run(
            [
                "aws",
                "s3",
                "sync",
                f"{local_dir}",
                f"{s3_uri}",
                "--exclude",
                "*.temp",
                "--exclude",
                "*.lock",
                "--exclude",
                "index.json",
                "--exclude",
                "*.tmp",
                "--exclude",
                "*.log",
            ],
            stdout=subprocess.DEVNULL,
            stderr=stderr,
            check=False,
        )
        time.sleep(upload_interval)

    subprocess.run(
        [
            "aws",
            "s3",
            "sync",
            f"{local_dir}",
            f"{s3_uri}",
            "--exclude",
            "*.temp",
            "--exclude",
            "*.lock",
            "--exclude",
            "*.tmp",
            "--exclude",
            "*.log",
        ],
        stdout=subprocess.DEVNULL,
        stderr=stderr,
        check=False,
    )

    if log_dir is not None:
        stderr.close()

def is_stable(path, wait=2):
    s1 = os.stat(path).st_size
    time.sleep(wait)
    s2 = os.stat(path).st_size
    return s1 == s2
    
def clean_up_local(
    s3_uri: str,
    local_dir: str | os.PathLike,
    delete_interval: int,
    stop_event: mp.Event,
    upload_done: mp.Event,
    log_dir: Optional[str | os.PathLike] = None
    ):
    if log_dir is None:
        stderr = subprocess.DEVNULL
    else:
        os.makedirs(log_dir, exist_ok=True)
        stderr = open(os.path.join(log_dir, "clean_up_logs.log"), "ab")
    while not stop_event.is_set():
        for file in os.listdir(local_dir):
            if not file.endswith(".zstd"):
                continue

            local_path = os.path.join(local_dir, file)
            try:
                # if not is_stable(local_path):
                #     continue
                res = subprocess.run(["aws", "s3", "ls", f"{s3_uri}{file}"], stdout=subprocess.DEVNULL, stderr=stderr)
                if res.returncode!=0:
                    continue
                os.remove(local_path)
            except FileNotFoundError:
                    # raced with writer or previous cleanup
                    pass
            except Exception as e:
                if log_dir is not None:
                    stderr.write(f"{file}: {str(e)}\n".encode())
        time.sleep(delete_interval)
    while not upload_done.is_set():
        time.sleep(0.5)    
    shutil.rmtree(local_dir, ignore_errors=True)
    if log_dir is not None:
        stderr.close()
    
        
    
def mds_write(
    root_dir: str | os.PathLike,
    shard_path: str | os.PathLike,
    compression: Literal["zstd"],
    logs_dir: Optional[str | os.PathLike],
    # stop_event : mp.Event,
    size_limit: str = "256mb",
    fps: int = 30,
    infer_class_from_path: bool = True,
    cls : Optional[int|Literal["real","fake"]] = None
) -> None:
    """
    Process raw videos and write them into MDS shards.

    Args:
        root_dir (str | os.PathLike): Directory containing raw videos.
        shard_path (str | os.PathLike): Output shard directory.
        compression (Literal["zstd"]): Compression algorithm.
        logs_dir (Optional[str | os.PathLike]): Directory for ffmpeg logs.
        stop_event (mp.Event): Event signaling uploader termination.
        size_limit (str): Maximum shard size.
        fps (int): Frames per second for encoding.
        infer_class_from_path (bool): Infer class from file path.
        cls (Optional[int | Literal['real','fake']]): Label given explicitly
    """
    if logs_dir is None:
        logs_dir = shard_path
    os.makedirs(logs_dir, exist_ok=True)
    ffmpeg_logs = open(os.path.join(logs_dir, "ffmpeg_logs.log"), "ab")

    def infer_class(path: str) -> int:
        if cls is not None and isinstance(cls, int): return cls
        elif cls is not None:
            if cls=='real':
                return 0
            else:
                return 1
        elif infer_class_from_path:
            if "fake" in path or "fakes" in path or "Fake" in path or "Fakes" in path:
                return 1
            else:
                return 0
        return 0

    temp_save_dir = os.path.join(tempfile.tempdir, "mks_save_dir")
    os.makedirs(temp_save_dir, exist_ok=True)

    columns = {
        "video": "bytes",
        "class": "int",
        "pad_inf": "ndarray",
        "actual_path": "str",
    }

    with MDSWriter(
        out=shard_path,
        columns=columns,
        compression=compression,
        size_limit=size_limit,
    ) as out:
        for video_name in tqdm.tqdm(os.listdir(root_dir)):
            try:
                if video_name.endswith((".mp4", ".avi", ".mks")):
                    video_path = os.path.join(root_dir, video_name)
                    cls = infer_class(video_path)
                    faces, fps = detect_faces(video_path, device="cuda")
                    frames = _get_frames(video_path)

                    crops = []
                    for idx in range(frames.shape[0]):
                        crops.append(_get_crop(frames[idx], faces[idx][0]))

                    n_crops, actual_h_w, h, w = pad_crops_to_max_dims(crops)

                    temp_save_path = (
                        str(temp_save_dir)
                        + f"/{video_name.split('.')[0]}.mkv"
                    )

                    proc = subprocess.Popen(
                        [
                            "ffmpeg",
                            "-y",
                            "-f",
                            "rawvideo",
                            "-pix_fmt",
                            "rgb24",
                            "-s",
                            f"{w}x{h}",
                            "-r",
                            str(fps),
                            "-i",
                            "-",
                            "-c:v",
                            "ffv1",
                            f"{temp_save_path}",
                        ],
                        stdin=subprocess.PIPE,
                        stdout=ffmpeg_logs,
                        stderr=ffmpeg_logs,
                    )

                    for frame in n_crops:
                        proc.stdin.write(frame.tobytes())

                    proc.stdin.close()
                    proc.wait()

                    with open(temp_save_path, "rb") as f:
                        video_bytes = f.read()

                    out.write(
                        {
                            "video": video_bytes,
                            "class": cls,
                            "pad_inf": actual_h_w,
                            "actual_path": str(video_path),
                        }
                    )

                    os.remove(temp_save_path)

            except Exception as e:
                with open(
                    os.path.join(logs_dir, f"{video_name}.txt"), "w"
                ) as f:
                    f.write(str(e))
                continue

    ffmpeg_logs.close()


def shard_upload(
    root_dir: str | os.PathLike,
    shard_dir: str | os.PathLike,
    s3_uri: Optional[str],
    log_dir: Optional[str | os.PathLike],
    compression: Literal["zstd"]="zstd",
    size_limit: str="256mb",
    fps: int=30,
    upload_interval: int=10,
    keep_local: bool = False,
    delete_interval: int = 20,
    infer_class_from_path: bool = True,
    cls : Optional[int|Literal["real","fake"]] = None
) -> None:
    """
    Entry point that coordinates shard writing and background S3 upload.

    Args:
        root_dir (str | os.PathLike): Directory with raw videos.
        shard_dir (str | os.PathLike): Directory for shard output.
        s3_uri (Optional[str]): S3 destination URI.
        compression (Literal["zstd"]): Compression algorithm.
        size_limit (str): Shard size limit.
        fps (int): Frames per second.
        upload_interval (int): S3 sync interval.
        log_dir (Optional[str | os.PathLike]): ffmpeg log directory.
    """
    stop_event = mp.Event()
    upload_done = None
    if s3_uri is not None:
        upload_done = mp.Event()
        uploader = mp.Process(
            target=upload_to_s3,
            args=(s3_uri, shard_dir, upload_interval, log_dir, stop_event),
            daemon=True,
        )
        uploader.start()
        
        if not keep_local:
            clean_up = mp.Process(
            target=clean_up_local,
            args=(s3_uri, shard_dir, delete_interval, stop_event, upload_done, log_dir),
            daemon=True
            )
            clean_up.start()
        else:
            clean_up = None
    else:
        uploader = None
        clean_up = None

    try:
        mds_write(
            root_dir=root_dir,
            shard_path=shard_dir,
            compression=compression,
            size_limit=size_limit,
            fps=fps,
            infer_class_from_path=True,
            logs_dir=log_dir,
            cls = cls
        )
    finally:
        if uploader is not None:
            stop_event.set()
            uploader.join()
            upload_done.set()
            if clean_up is not None:
                clean_up.join()

if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description="Write video datasets into MDS shards with optional S3 upload."
    )
    
    
    parser.add_argument("--root_dir", type=str, required=True)
    parser.add_argument("--shard_dir", type=str, required=True)

    parser.add_argument("--s3_uri", type=str, default=None)

    parser.add_argument("--compression", type=str, default="zstd")
    parser.add_argument("--size_limit", type=str, default="256mb")
    parser.add_argument("--fps", type=int, default=25)
    parser.add_argument("--upload_interval", type=int, default=5)
    parser.add_argument("--delete_interval", type=int, default=2)
    parser.add_argument("--keep_local", type=bool, default=False)
    parser.add_argument("--cls", type=str, default=None)
    parser.add_argument(
        "--infer_class_from_path",
        action="store_true",
        default=True
    )
    
    parser.add_argument("--log_dir", type=str, default=None)

    args = parser.parse_args()

    shard_upload(
        root_dir=args.root_dir,
        shard_dir=args.shard_dir,
        s3_uri=args.s3_uri,
        compression=args.compression,
        size_limit=args.size_limit,
        fps=args.fps,
        upload_interval=args.upload_interval,
        keep_local = args.keep_local,
        delete_interval = args.delete_interval,
        log_dir=args.log_dir,
        infer_class_from_path=args.infer_class_from_path,
        cls = args.cls
    )

    
